import json

class Alfred(dict):

    def __init__(self):
        dict.__init__(self, items=[])

    def add_item(self, item):
        self['items'].append(item)

    def alphy(self):
        return json.dumps(self, ensure_ascii=False)

class AlfredItem(dict):

    def __init__(self, title, subtitle, arg):
        dict.__init__(self,
            title=title,
            subtitle=subtitle,
            arg=arg,
            text={},
        )

    def set_clipboard_text(self, text):
        self['text']['copy'] = text

    def set_largetype_text(self, text):
        self['text']['largetype'] = text

def alphy(search, variations) -> str:
    alfred = Alfred()

    for v in variations:
        name = v.name
        for m in v.meanings:
            title       = m.text
            subtitle    = ', '.join(m.props)
            arg         = search

            if v.origin:
                subtitle = "{} - {}".format(v.origin, subtitle)

            sample = ''
            for s in m.samples:
                sample += '- {}'.format(s.text)
                if s.authors:
                    sample += '\\n'
                    sample += ', '.join(s.authors)
                break

            largetype = '{}\\n{}\\n\\n{}\\n\\n{}'.format(name, subtitle, title, sample)

            item = AlfredItem(title, subtitle, arg)

            item.set_clipboard_text(title)
            item.set_largetype_text(largetype)

            alfred.add_item(item)

    return alfred.alphy()




from termcolor import colored

def colorize(variations, verbose=False):
    text = ''
    for i, v in enumerate(variations, start=1):
        text += colored(v.name, 'yellow', attrs=['bold', 'underline'])
        if len(variations) > 2:
            text += ' {}'.format(colored('[{}]'.format(i), attrs=['dark']))
        if v.origin:
            text += ' - ' + colored(v.origin, 'yellow', attrs=['dark'])

        text += '\n\n'
        for m in v.meanings:
            text += '• ' + m.text + ' '

            if m.props:
                text += colored(
                    ', '.join(m.props),
                    'yellow', attrs=['dark'],
                )

            text += '\n'
            for s in m.samples:
                text += '  {}  '.format(colored('Örnek', 'magenta'))
                text += '  {}'.format(colored(s.text, 'green'))
                if s.authors:
                    text += '\n'
                    text += '  {} '.format(colored('Yazarlar', 'magenta'))
                    text += colored(', '.join(s.authors), 'blue')
                text += '\n'
            else:
                text += '\n'

        if not verbose:
            continue

        if v.compound_words:
            text += colored('Birleşik Kelimeler', 'cyan') + '\n'
            text += '\n'.join(['• ' + c for c in v.compound_words])
            text += '\n\n'

        if v.proverbs:
            text += colored(
                'Atasözleri, Deyimler veya Birleşik Fiiller', 'cyan'
            )
            text += '\n'
            text += '\n'.join(['• ' + p for p in v.proverbs])
            text += '\n'

    return text


import requests

from lugat.__version__ import __version__
from lugat.word import new as new_word

class LookupException(Exception):
    pass


URL = 'https://sozluk.gov.tr/gts?ara={}'
USER_AGENT = ('lugat/' + __version__)
REFERER = 'github.com/ademozay/lugat'
HEADERS = {
    'User-Agent': USER_AGENT,
    'Referer': REFERER,
}


def lookup(search):
    if search is None:
        return None

    try:
        response = requests.get(URL.format(search), headers=HEADERS)
        result = response.json()

        if 'error' in result:
            return None

        return new_word(result)
    except Exception as e:
        raise LookupException('lookup error') from e



import sys

from lugat.__version__ import __version__

from lugat.dict import lookup
from lugat.alfred import alphy
from lugat.colorize import colorize
from lugat.args import Args

def main():
    args    = Args()
    search  = args.get_input()
    word    = lookup(search)

    if word is None:
        exit(0)

    variations = word.get_variations()

    if args.is_enabled(Args.Flag.ALFRED):
        out = alphy(search, variations)
        sys.stdout.write(out)
    else:
        verbose = args.is_enabled(Args.Flag.VERBOSE)
        out = colorize(variations, verbose)
        sys.stdout.write(out)

if __name__ == "__main__":
    main()



import sys
import unicodedata

from enum import Enum

from termcolor import colored

from lugat.__version__ import __version__

HELP_TEXT = '''

    ''' + colored("TDK Güncel Türkçe Sözlük'te arama yapın.", attrs=['bold']) + '''

        ''' + colored('lugat', attrs=['bold']) + ''' mütevazı
        ''' + colored('lugat', attrs=['bold']) + ''' cefakâr

    Daha detaylı sonuçlar için:

        ''' + colored('lugat', attrs=['bold']) + ''' -h çırak
        ''' + colored('lugat', attrs=['bold']) + ''' -h usta

    * Detaylı sonuçlar; atasözleri, deyimler, 
    birleşik fiiller ve birleşik kelimelerden oluşur.

    lugat ''' +  colored('v' + __version__, attrs=['bold'])  + '''
'''

class Args:

    class Flag(Enum):
        VERBOSE    = '-h'
        ALFRED     = '--alfred'

    flags           = [f.value for f in Flag]
    enabled_flags   = []
    input           = ''

    def __init__(self):
        args = sys.argv[1:]
        if len(args) < 1:
            sys.stdout.write(HELP_TEXT)
            exit(2)

        self._resolve_flags(args)
        self._resolve_input(args)

    def is_enabled(self, flag):
        return flag.value in self.enabled_flags

    def get_input(self):
        return self.input

    def _resolve_flags(self, args):
        for arg in args:
            for flag in self.flags:
                if flag == arg:
                    self.enabled_flags.append(flag)

    def _resolve_input(self, args):
        remaing_args    = list(filter(lambda arg: arg not in self.flags, args))
        input           = ' '.join(remaing_args)
        self.input      = self._normalize(input)

        if not self.input:
            sys.stdout.write(HELP_TEXT)
            sys.exit(2)

    def _normalize(self, text, normalization='NFC'):
        # https://www.alfredforum.com/topic/7357-encoding-in-script-filter-input/?tab=comments#comment-39232
        return unicodedata.normalize(normalization, text)